/**
 * 
 */
package eu.etaxonomy.taxeditor.editor.descriptiontree.handler;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.expressions.PropertyTester;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.jface.viewers.TreeSelection;

import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.media.ImageFile;

/**
 * @author p.ciardelli
 *
 */
public class DescriptionsMenuPropertyTester extends PropertyTester {

	private List segments;

	/* (non-Javadoc)
	 * @see org.eclipse.core.expressions.IPropertyTester#test(java.lang.Object, java.lang.String, java.lang.Object[], java.lang.Object)
	 */
	public boolean test(Object receiver, String property, Object[] args,
			Object expectedValue) {
		
		segments = getSegments((TreeSelection) receiver);

		if ("isInImageGallery".equals(property)) {
			return isInImageGallery();
		}
		
		if ("isInImageList".equals(property)) {
			return isInImageGallery() && isInDescriptionElement();
		}
		
		if ("isImage".equals(property)) {
			return isImage();
		}
		
		if ("isInDescription".equals(property)) {
			return isInDescription();
		}
		
		if ("isNonImageElement".equals(property)) {
			return isInDescription() && isInDescriptionElement();
		}
		if("isDescription".equals(property)){
			return isDescription(); 
		}
		
		return false;
	}
	
	/**
	 * @return
	 */
	private boolean isDescription() {
		for (Object o : segments) {
			if(!(o instanceof DescriptionBase))
				return false;
		}
		return true;
	}

	/**
	 * @param receiver
	 * @return
	 */
	private List getSegments(TreeSelection selection) {
		List list = new ArrayList();
		if(selection.getPaths().length > 0){
			TreePath segmentsPath = selection.getPaths()[0];
			for (int i = 0; i < segmentsPath.getSegmentCount(); i++) {
				list.add(segmentsPath.getSegment(i));
			}		
		}
		return list;
	}

	
	
	/**
	 * @return
	 */
	private boolean isInDescription() {
		for (Object o : segments) {
			if (o instanceof TaxonDescription) {
				return !((TaxonDescription) o).isImageGallery();
			}
		}
		return false;
	}

	/**
	 * @return
	 */
	private boolean isImage() {
		for (Object o : segments) {
			if (o instanceof ImageFile) {
				return true;
			}
		}
		return false;
	}

	private boolean isInImageGallery() {
		for (Object o : segments) {
			if (o instanceof TaxonDescription) {
				return ((TaxonDescription) o).isImageGallery();
			}
		}
		return false;
	}

	private boolean isInDescriptionElement() {
		for (Object o : segments) {
			if (o instanceof DescriptionElementBase) {
				return true;
			}
		}
		return false;		
	}
	
}
