// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.forms;

import org.apache.log4j.Logger;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;

import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.cdm.model.reference.ReferenceType;
import eu.etaxonomy.taxeditor.forms.selectioncomposites.AgentSelectionComposite;
import eu.etaxonomy.taxeditor.forms.selectioncomposites.InstitutionSelectionComposite;
import eu.etaxonomy.taxeditor.forms.selectioncomposites.ReferenceSelectionComposite;
import eu.etaxonomy.taxeditor.forms.termcombos.ReferenceTypeComboComposite;

/**
 * @author n.hoffmann
 * @created Nov 17, 2009
 * @version 1.0
 */
public class ReferenceBaseDetailComposite extends AbstractFormComposite {
	private static final Logger logger = Logger
			.getLogger(ReferenceBaseDetailComposite.class);
	private ReferenceBase referenceBase;
	private Composite composite_referenceParts;
	private TextWithLabelComposite text_edition;
	private TextWithLabelComposite text_editor;
	private TextWithLabelComposite text_isbn;
	private TextWithLabelComposite text_issn;
	private TextWithLabelComposite text_organisation;
	private TextWithLabelComposite text_pages;
	private TextWithLabelComposite text_placePublished;
	private TextWithLabelComposite text_publisher;
	private MultilineTextWithLabel text_referenceAbstract;
	private InstitutionSelectionComposite selection_school;
	private TextWithLabelComposite text_series;
	private TextWithLabelComposite text_seriesPart;
	private TextWithLabelComposite text_title;
	private TextWithLabelComposite text_volume;
	private TextWithLabelComposite text_uri;
	private AgentSelectionComposite selection_agent;
	private TimePeriodComposite composite_timePeriod;
	private ReferenceSelectionComposite selection_inReference;
	private InstitutionSelectionComposite selection_institution;
	private ReferenceTypeComboComposite combo_referenceType;
	private ReferenceType referenceType;
	
	public void setInput(ReferenceBase referenceBase){
		this.referenceBase = referenceBase;
		referenceType = referenceBase.getType();
			
		createControls(this, this, SWT.NULL);
	}


	public ReferenceBaseDetailComposite(Composite parent,
			IPropertyChangeListener listener, int containerStyle){
		super(parent, listener, containerStyle);
	}

	public void setEnabled(boolean enabled){
		if(composite_referenceParts != null){
			for(Control control : composite_referenceParts.getChildren()){
				if(control instanceof Composite){
					((Composite) control).setEnabled(enabled);
				}
			}
		}
	}
	
	private void createControls(Composite parent, IPropertyChangeListener listener, int style){
		if(composite_referenceParts != null)
			composite_referenceParts.dispose();
		
		composite_referenceParts = toolkit.createComposite(parent);
		composite_referenceParts.setLayout(new GridLayout(2, false));
		composite_referenceParts.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
		
		// title
		toolkit.createLabel(composite_referenceParts, "Title");
		text_title = toolkit.createTextComposite(composite_referenceParts, referenceBase.getTitle(), listener, style);
		text_title.setLayoutData(FILL_GRID_HORIZONTALLY());
		
		// author team
		toolkit.createLabel(composite_referenceParts, "Author Team");
		selection_agent = new AgentSelectionComposite(composite_referenceParts, null, referenceBase.getAuthorTeam(), listener, style);
		
		// date published
		toolkit.createLabel(composite_referenceParts, "Date Published");
		composite_timePeriod = new TimePeriodComposite(composite_referenceParts, referenceBase.getDatePublished(), listener, style);
		
		/* ReferenceBase END */
		
		/* Generic */
		if(referenceType != null && referenceType.equals(ReferenceType.Generic)){
			createGenericControls(composite_referenceParts, listener, style);
		}else if(referenceType != null){
		
			createPublicationControls(composite_referenceParts, listener, style);

			createVolumeControls(composite_referenceParts, listener, style);

			createSectionControls(composite_referenceParts, listener, style);

			createPrintedUnitControls(composite_referenceParts, listener, style);				
			
		}
		
		// uri
		toolkit.createLabel(composite_referenceParts, "URI");
		text_uri = toolkit.createTextComposite(composite_referenceParts, referenceBase.getUri(), listener, style);
		
		
		// reference abstract
		text_referenceAbstract = toolkit.createMultilineTextWithLabel(composite_referenceParts, "Reference Abstract", listener, 100, style);
		text_referenceAbstract.setText(referenceBase.getReferenceAbstract());
		text_referenceAbstract.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 2, 1));
		
		parent.layout();
	}
	
	/**
	 * Creates additional controls to edit a generic reference 
	 * 
	 * @param parent
	 * @param listener
	 * @param style
	 */
	private void createGenericControls(Composite parent, IPropertyChangeListener listener, int style){
		//editor
		toolkit.createLabel(parent, "Editor");
		text_editor = toolkit.createTextComposite(parent, referenceBase.getEditor(), listener, style);
		
		// series
		toolkit.createLabel(parent, "Series");
		text_series = toolkit.createTextComposite(parent, referenceBase.getSeries(), listener, style);
		
		// volume 
		toolkit.createLabel(parent, "Volume");
		text_volume = toolkit.createTextComposite(parent, referenceBase.getVolume(), listener, style);
		
		// pages
		toolkit.createLabel(parent, "Pages");
		text_pages = toolkit.createTextComposite(parent, referenceBase.getPages(), listener, style);
	}
	
	/**
	 * Creates addition controls to edit a publication
	 * 
	 * @param parent
	 * @param listener
	 * @param style
	 */
	private void createPublicationControls(Composite parent, IPropertyChangeListener listener, int style){
		if(! referenceType.isPublication()){
			return;
		}
		
		// placePublished
		toolkit.createLabel(parent, "Place Published");
		text_placePublished = toolkit.createTextComposite(parent, referenceBase.getPlacePublished(), listener, style);
		
		// publisher
		toolkit.createLabel(parent, "Publisher");
		text_publisher = toolkit.createTextComposite(parent, referenceBase.getPublisher(), listener, style);
		
		if(referenceType.equals(ReferenceType.Journal)){
			//issn
			toolkit.createLabel(parent, "ISSN");
			text_issn = toolkit.createTextComposite(parent, referenceBase.getIssn(), listener, style);
		}
		
		if(referenceType.equals(ReferenceType.Report)){
			// institution
			toolkit.createLabel(parent, "Institution");
			selection_institution = new InstitutionSelectionComposite(parent, null, referenceBase.getInstitution(), listener, style);
		}
		
		if(referenceType.equals(ReferenceType.Thesis)){
			// school
			toolkit.createLabel(parent, "School");
			selection_school = new InstitutionSelectionComposite(parent, null, referenceBase.getSchool(), listener, style);
		}
		
		if(referenceType.equals(ReferenceType.Proceedings)){
			//organization
			toolkit.createLabel(parent, "Organisation");
			text_organisation = toolkit.createTextComposite(parent, referenceBase.getOrganization(), listener, style);
		}
	}
	
	private void createSectionControls(Composite parent, IPropertyChangeListener listener, int style){
		if(! referenceType.isSection()){
			return;
		}
		
		// pages
		toolkit.createLabel(parent, "Pages");
		text_pages = toolkit.createTextComposite(parent, referenceBase.getPages(), listener, style);
		
		// in reference
		toolkit.createLabel(parent, "In Reference"); // TODO Label May differ according to ReferenceType
		selection_inReference = new ReferenceSelectionComposite(parent, null, referenceBase.getInReference(), listener, style);
	}
	
	private void createVolumeControls(Composite parent, IPropertyChangeListener listener, int style){
		if(!referenceType.isVolumeReference()){
			return;
		}
		
		// volume 
		toolkit.createLabel(parent, "Volume");
		text_volume = toolkit.createTextComposite(parent, referenceBase.getVolume(), listener, style);		
	}
	
	private void createPrintedUnitControls(Composite parent, IPropertyChangeListener listener, int style){
		if(! referenceType.isPrintedUnit()){
			return;
		}
		
		//editor
		toolkit.createLabel(parent, "Editor");
		text_editor = toolkit.createTextComposite(parent, referenceBase.getEditor(), listener, style);
		
		// series part
		toolkit.createLabel(parent, "Series Part");
		text_seriesPart = toolkit.createTextComposite(parent, referenceBase.getSeriesPart(), listener, style);
		
		if(referenceType.equals(ReferenceType.Book)){
			// edition
			toolkit.createLabel(parent, "Edition");
			text_edition = toolkit.createTextComposite(parent, referenceBase.getEdition(), listener, style);
			
			//isbn
			toolkit.createLabel(parent, "ISBN");
			text_isbn = toolkit.createTextComposite(parent, referenceBase.getIsbn(), listener, style);
		}
	}


	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.AbstractFormComposite#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	@Override
	public void propertyChange(PropertyChangeEvent event) {
		
		
		
		super.propertyChange(event);
	}

}
