// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.forms.entitysections;

import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.name.NomenclaturalStatus;
import eu.etaxonomy.cdm.model.name.TaxonNameBase;

/**
 * @author n.hoffmann
 * @created Nov 5, 2009
 * @version 1.0
 */
public class NomenclaturalStatusSection extends AbstractEntitySetSection<TaxonNameBase> {
	/**
	 * @param parent
	 * @param entity
	 * @param title
	 * @param listener
	 * @param style
	 */
	public NomenclaturalStatusSection(Composite parent,
			TaxonNameBase entity, IPropertyChangeListener listener, int style) {
		super(parent, entity, "Nomenclatural Status", listener, style);
	}

	private static final Logger logger = Logger
			.getLogger(NomenclaturalStatusSection.class);

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.entitysections.AbstractEntitySetSection#createDynamicContents()
	 */
	@Override
	protected void createDynamicContents() {
		Set<NomenclaturalStatus> nomenclaturalStatusSet = null;
		if(getEntity() != null){
			nomenclaturalStatusSet = getEntity().getStatus();
		}
		
		if(nomenclaturalStatusSet == null || nomenclaturalStatusSet.size() == 0){
			toolkit.createLabel(container, "No nomenclatural status yet.");
		}else{
			for(NomenclaturalStatus nomenclaturalStatus : nomenclaturalStatusSet){
				new NomenclaturalStatusComposite(container, nomenclaturalStatus, new RemoveListener(nomenclaturalStatus), this, SWT.NULL);
			}
		}
	}
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.entitysections.AbstractEntitySetSection#getAddAction()
	 */
	@Override
	protected IAction getAddAction() {
		Action addNomenclaturalStatus = new Action("add", Action.AS_PUSH_BUTTON){
			/* (non-Javadoc)
			 * @see org.eclipse.jface.action.Action#run()
			 */
			@Override
			public void run() {
				NomenclaturalStatus nomenclaturalStatus = NomenclaturalStatus.NewInstance(null);
				getEntity().addStatus(nomenclaturalStatus);
				if(! getSection().isExpanded())
					getSection().setExpanded(true);
				internalUpdateSection(true);
			}
		};
		addNomenclaturalStatus.setToolTipText("Create a new nomenclatural status");
		return addNomenclaturalStatus;
	}
	
	private class RemoveListener extends SelectionAdapter{
		private NomenclaturalStatus nomenclaturalStatus;
		
		public RemoveListener(NomenclaturalStatus nomenclaturalStatus){
			this.nomenclaturalStatus = nomenclaturalStatus;
		}
		
		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
		 */
		@Override
		public void widgetSelected(SelectionEvent e) {
			getEntity().removeStatus(nomenclaturalStatus);
			internalUpdateSection(true);
		}
	}
}
