// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.forms.entitysections;

import org.apache.log4j.Logger;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.jface.util.PropertyChangeEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.media.MediaRepresentation;
import eu.etaxonomy.cdm.model.media.MediaRepresentationPart;
import eu.etaxonomy.taxeditor.forms.TextWithLabelComposite;

/**
 * @author n.hoffmann
 * @created Nov 5, 2009
 * @version 1.0
 */
public class ProtologComposite extends AbstractEntitySetElementComposite<DescriptionElementBase> {

	/**
	 * @param parent
	 * @param element
	 * @param removeListener
	 * @param propertyChangelistener
	 * @param style
	 */
	public ProtologComposite(Composite parent, DescriptionElementBase element,
			SelectionListener removeListener,
			IPropertyChangeListener propertyChangelistener, int style) {
		super(parent, element, removeListener, propertyChangelistener, style);
	}

	private static final Logger logger = Logger
			.getLogger(ProtologComposite.class);

	private TextWithLabelComposite protologUriText;
	private MediaRepresentationPart mediaRepresentationPart;
	private Media media;
	private MediaRepresentation mediaRepresentation;
	
	
	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.AbstractFormComposite#propertyChange(org.eclipse.jface.util.PropertyChangeEvent)
	 */
	@Override
	public void propertyChange(PropertyChangeEvent event) {
		mediaRepresentationPart.setUri(protologUriText.getText());		
		
		super.propertyChange(event);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.entitysections.AbstractEntitySetElementComposite#createControls(org.eclipse.swt.widgets.Composite, int)
	 */
	@Override
	public void createControls(Composite parent, int style) {
		protologUriText = toolkit.createTextWithLabelComposite(parent, "Protolog URI", "", this, style);
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.entitysections.AbstractEntitySetElementComposite#setElement(eu.etaxonomy.cdm.model.common.VersionableEntity)
	 */
	@Override
	public void setElement(DescriptionElementBase element) {
		this.element = element;
		
		if(element.getMedia().size() == 0){
			element.addMedia(Media.NewInstance());
		}
		media = element.getMedia().iterator().next();
		
		
		if(media.getRepresentations().size() == 0){
			media.addRepresentation(MediaRepresentation.NewInstance());
		}		
		mediaRepresentation = media.getRepresentations().iterator().next();
		
		if(mediaRepresentation.getParts().size() == 0){
			mediaRepresentation.addRepresentationPart(MediaRepresentationPart.NewInstance(null, null));
		}
		
		mediaRepresentationPart = mediaRepresentation.getParts().iterator().next();
				
		protologUriText.setText(mediaRepresentationPart.getUri());
	}
}
