// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.forms.entitysections;

import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.common.Annotation;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.TaxonNameDescription;
import eu.etaxonomy.cdm.model.description.TextData;

/**
 * @author n.hoffmann
 * @created Nov 5, 2009
 * @version 1.0
 */
public class ProtologSection extends AbstractEntitySetSection<TaxonNameDescription> {
	
	private static final Logger logger = Logger
	.getLogger(ProtologSection.class);

	/**
	 * @param parent
	 * @param entity
	 * @param title
	 * @param listener
	 * @param style
	 */
	public ProtologSection(Composite parent, TaxonNameDescription entity,
			IPropertyChangeListener listener, int style) {
		super(parent, entity, "Protologs", listener, style);
	}


	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.entitysections.AbstractEntitySetSection#createDynamicContents()
	 */
	@Override
	protected void createDynamicContents() {
		Set<DescriptionElementBase> descriptions = null;
		if(getEntity() != null){
			descriptions = getEntity().getElements();
		}
		
		if(descriptions == null || descriptions.size() == 0){
			toolkit.createLabel(container, "No protologs yet.");
		}else{
			for(DescriptionElementBase description : descriptions){
				if(description.getFeature().equals(Feature.PROTOLOG())){
					new ProtologComposite(container, description, new RemoveListener(description), this, SWT.NULL);
				}
			}
		}
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.forms.entitysections.AbstractEntitySetSection#getAddAction()
	 */
	@Override
	protected IAction getAddAction() {
		Action addProtolog = new Action("add", Action.AS_PUSH_BUTTON){
			/* (non-Javadoc)
			 * @see org.eclipse.jface.action.Action#run()
			 */
			@Override
			public void run() {
				getEntity().addElement(TextData.NewInstance(Feature.PROTOLOG()));
				
				if(! getSection().isExpanded())
					getSection().setExpanded(true);
				internalUpdateSection(true);
			}
		};
		addProtolog.setToolTipText("Create a new protolog");
		return addProtolog;
	}
	
	
	private class RemoveListener extends SelectionAdapter{
		private DescriptionElementBase description;
		
		public RemoveListener(DescriptionElementBase description){
			this.description = description;
		}
		
		/* (non-Javadoc)
		 * @see org.eclipse.swt.events.SelectionAdapter#widgetSelected(org.eclipse.swt.events.SelectionEvent)
		 */
		@Override
		public void widgetSelected(SelectionEvent e) {
			getEntity().removeElement(description);
			internalUpdateSection(true);
		}
	}
}
