// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.forms.termcombos;

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.eclipse.jface.util.IPropertyChangeListener;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;

import eu.etaxonomy.cdm.model.reference.ReferenceType;
import eu.etaxonomy.taxeditor.forms.AbstractFormComposite;
import eu.etaxonomy.taxeditor.store.CdmStore;

/**
 * @author n.hoffmann
 * @created Nov 16, 2009
 * @version 1.0
 */
public class ReferenceTypeComboComposite extends AbstractFormComposite implements SelectionListener{
	private static final Logger logger = Logger
			.getLogger(ReferenceTypeComboComposite.class);

	private static final int VISIBLE_ITEMS = 5;
	
	private ReferenceType selection;
	
	private Combo combo;

	private List<ReferenceType> referenceTypeList;
	
	/**
	 * @param parent
	 * @param listener
	 * @param style
	 */
	public ReferenceTypeComboComposite(Composite parent, ReferenceType referenceType,
			IPropertyChangeListener listener, int style) {
		super(parent, listener, style);
		// create combo
	    combo = new Combo(this, SWT.BORDER | SWT.READ_ONLY);
	    combo.setLayoutData(new GridData(SWT.FILL, SWT.TOP, true, false));
	    combo.setVisibleItemCount(VISIBLE_ITEMS);
	    
	    referenceTypeList = new ArrayList<ReferenceType>();
	    
	    populateTypes();
	    
	    combo.addSelectionListener(this);
	    setSelection(referenceType);
	}

	/**
	 * 
	 */
	private void populateTypes() {
		for(ReferenceType referenceType : ReferenceType.values()){
			referenceTypeList.add(referenceType);
			combo.add(referenceType.getMessage(CdmStore.getDefaultLanguage()));
		}
	}
	
	

	/* (non-Javadoc)
	 * @see org.eclipse.swt.events.SelectionListener#widgetSelected(org.eclipse.swt.events.SelectionEvent)
	 */
	public void widgetSelected(SelectionEvent e) {
		selection = referenceTypeList.get(combo.getSelectionIndex());
	}

	public void widgetDefaultSelected(SelectionEvent e) {}

	/**
	 * @param selection the selection to set
	 */
	public void setSelection(ReferenceType selection) {
		this.selection = selection;
		combo.select(referenceTypeList.indexOf(selection));
	}
	
	public void addSelectionListener(SelectionListener listener){
		combo.addSelectionListener(listener);
	}
	
	public void removeSelectionListener(SelectionListener listener){
		combo.removeSelectionListener(listener);
	}

	/**
	 * @return the selection
	 */
	public ReferenceType getSelection() {
		return selection;
	}
}
