/**
 * 
 */
package eu.etaxonomy.taxeditor.propertysheet.authors;

import java.util.Vector;

import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.agent.Person;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.taxeditor.propertysheet.ICdmBasePropertySource;

/**
 * @author p.ciardelli
 *
 */
public class PersonPropertySource implements ICdmBasePropertySource {

	private Person person;
	
	public static final String P_ID_TITLE = "P_ID_TITLE";
	public static final String P_ID_NOM_TITLE = "P_ID_NOM_TITLE";
	public static final String P_ID_PROTECT_CACHE = "P_ID_PROTECT_CACHE";
	public static final String P_ID_PREFIX = "P_ID_PREFIX";
	public static final String P_ID_FIRSTNAME = "P_ID_FIRSTNAME";
	public static final String P_ID_LASTNAME = "P_ID_LASTNAME";
	public static final String P_ID_SUFFIX = "P_ID_SUFFIX";

	public static final String P_TITLE = "Editable title cache";
	public static final String P_NOM_TITLE = "Nomenclatural title";
	public static final String P_PROTECT_CACHE = "Protect title cache?";
	public static final String P_PREFIX = "Prefix";
	public static final String P_FIRSTNAME = "First name";
	public static final String P_LASTNAME = "Last name";
	public static final String P_SUFFIX = "Suffix";

	private static final int CACHE_NOT_PROTECTED = 0;
	private static final int CACHE_PROTECTED = 1;
	
	public PersonPropertySource(Person person) {
		this.person = person;
		
		initDescriptors();
	}
	
	/**
	 * 
	 */
	private void initDescriptors() {
		addDescriptor(P_ID_TITLE);
		addDescriptor(P_ID_PROTECT_CACHE);
		addDescriptor(P_ID_NOM_TITLE);
		addDescriptor(P_ID_PREFIX);
		addDescriptor(P_ID_FIRSTNAME);
		addDescriptor(P_ID_LASTNAME);
		addDescriptor(P_ID_SUFFIX);
	}

	protected Vector<PropertyDescriptor> descriptors = new Vector<PropertyDescriptor>();
	
	/**
	 * @param field
	 */
	private void addDescriptor(String id) {
		
		// Editable cache
		if (id.equals(P_ID_TITLE)) {
			descriptors.addElement(
					new TextPropertyDescriptor(P_ID_TITLE, P_TITLE));
		}
		
		// Editable cache
		if (id.equals(P_ID_NOM_TITLE)) {
			descriptors.addElement(
					new TextPropertyDescriptor(P_ID_NOM_TITLE, P_NOM_TITLE));
		}
		
		// Protect cache?
		if (id.equals(P_ID_PROTECT_CACHE)) {
			descriptors.addElement(
					new ComboBoxPropertyDescriptor(P_ID_PROTECT_CACHE, P_PROTECT_CACHE, 
							new String[] {"no", "yes"}));			
		}
		
		boolean isProtectedCache = person.isProtectedTitleCache();
		
		if (id.equals(P_ID_PREFIX)) {
			descriptors.addElement(
					isProtectedCache?
							new PropertyDescriptor(P_ID_PREFIX, P_PREFIX) :
							new TextPropertyDescriptor(P_ID_PREFIX, P_PREFIX));			
		}
		if (id.equals(P_ID_FIRSTNAME)) {
			descriptors.addElement(
					isProtectedCache?
							new PropertyDescriptor(P_ID_FIRSTNAME, P_FIRSTNAME) :
							new TextPropertyDescriptor(P_ID_FIRSTNAME, P_FIRSTNAME));			
		}
		if (id.equals(P_ID_LASTNAME)) {
			descriptors.addElement(
					isProtectedCache?
							new PropertyDescriptor(P_ID_LASTNAME, P_LASTNAME) :
							new TextPropertyDescriptor(P_ID_LASTNAME, P_LASTNAME));			
		}
		if (id.equals(P_ID_SUFFIX)) {
			descriptors.addElement(
					isProtectedCache?
							new PropertyDescriptor(P_ID_SUFFIX, P_SUFFIX) :
							new TextPropertyDescriptor(P_ID_SUFFIX, P_SUFFIX));			
		}
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
		return (IPropertyDescriptor[]) descriptors.toArray(
                new IPropertyDescriptor[descriptors.size()]);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
	 */
	public Object getEditableValue() {
		// TODO Auto-generated method stub
		return null;
	}


	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(Object id) {
		
		if (id.equals(P_ID_TITLE)) {
			return CdmUtils.Nz(person.getTitleCache());
		}

		if (id.equals(P_ID_NOM_TITLE)) {
			return CdmUtils.Nz(person.getNomenclaturalTitle());
		}
		
		if (id.equals(P_ID_PROTECT_CACHE)) {
			if (person.isProtectedTitleCache()) {
				return CACHE_PROTECTED;
			} else {
				return CACHE_NOT_PROTECTED;
			}
		}
		
		if (id.equals(P_ID_PREFIX)) {
			return CdmUtils.Nz(person.getPrefix());
		}
		if (id.equals(P_ID_FIRSTNAME)) {
			return CdmUtils.Nz(person.getFirstname());
		}
		if (id.equals(P_ID_LASTNAME)) {
			return CdmUtils.Nz(person.getLastname());
		}
		if (id.equals(P_ID_SUFFIX)) {
			return CdmUtils.Nz(person.getSuffix());
		}
		
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
	 */
	public boolean isPropertySet(Object id) {
		// TODO Auto-generated method stub
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
	 */
	public void resetPropertyValue(Object id) {
		// TODO Auto-generated method stub

	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(Object id, Object value) {
		
		if (id.equals(P_ID_TITLE)) {
			person.setTitleCache((String) value);
		}

		if (id.equals(P_ID_NOM_TITLE)) {
			person.setNomenclaturalTitle((String) value);
		}
		
		// Protect cache? 
		if (id.equals(P_ID_PROTECT_CACHE)) {
			if (((Integer) value).intValue() == CACHE_PROTECTED) {
				person.setProtectedTitleCache(true);
			} else {
				person.setProtectedTitleCache(false);
			}
		}
		
		if (id.equals(P_ID_PREFIX)) {
			person.setPrefix((String) value);
		}
		if (id.equals(P_ID_FIRSTNAME)) {
			person.setFirstname((String) value);
		}
		if (id.equals(P_ID_LASTNAME)) {
			person.setLastname((String) value);
		}
		if (id.equals(P_ID_SUFFIX)) {
			person.setSuffix((String) value);
		}
	}

	/**
	 * @return
	 */
	public CdmBase getPerson() {
		return person;
	}

}
