// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.type;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.Vector;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.common.IdentifiableEntity;
import eu.etaxonomy.cdm.model.name.NameTypeDesignation;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignation;
import eu.etaxonomy.cdm.model.name.TypeDesignationBase;
import eu.etaxonomy.cdm.model.reference.Generic;
import eu.etaxonomy.cdm.model.reference.ReferenceBase;
import eu.etaxonomy.taxeditor.propertysheet.ICdmBasePropertySource;
import eu.etaxonomy.taxeditor.propertysheet.reference.ReferencePropertySource;

/**
 * @author p.ciardelli
 * @created 28.04.2009
 * @version 1.0
 */
public class TypeDesignationPropertySource implements ICdmBasePropertySource {

	private TypeDesignationBase<?> typeDesignation;

    /**
     * Property unique keys
     */
	public static final String P_ID_REFERENCE = "reference";
	public static final String P_ID_REFDETAIL = "refDetail";

    /**
     * Property display keys
	 */
	public static final String P_REFERENCE = "Reference";
	public static final String P_REFDETAIL = "Reference Detail";
	
	protected Vector<PropertyDescriptor> descriptors = new Vector<PropertyDescriptor>();
	
	TypeDesignationPropertySource(TypeDesignationBase<?> typeDesignation) {
		this.typeDesignation = typeDesignation;
		
		addDescriptor(P_ID_REFERENCE);
		addDescriptor(P_ID_REFDETAIL);
	}
	
	public Object getEditableValue() {		
		String str = "";
		IdentifiableEntity identifiableEntity;
		if (typeDesignation instanceof SpecimenTypeDesignation) {
			identifiableEntity = ((SpecimenTypeDesignation) typeDesignation).getTypeSpecimen();
		} else {
			identifiableEntity = ((NameTypeDesignation) typeDesignation).getTypeName();
		}
		if (identifiableEntity != null) {
			str = identifiableEntity.getTitleCache();
		} else {
			str = "UNDEFINED";
		}
		if (typeDesignation instanceof NameTypeDesignation && ((NameTypeDesignation) typeDesignation).isNotDesignated()) {
			return str + " [not designated]";
		}
		if (typeDesignation.getTypeStatus() != null) {
			return str + " [" + typeDesignation.getTypeStatus().getLabel() + "]";
		}
		return str;
	}

	private void addDescriptor(String id) {
		// Name relations, listed in custom property descriptor
		if (id.equals(P_ID_REFERENCE)) {
			descriptors.addElement(
					new PropertyDescriptor(P_ID_REFERENCE, P_REFERENCE));
		}
		if (id.equals(P_ID_REFDETAIL)) {
			descriptors.addElement(
					new TextPropertyDescriptor(P_ID_REFDETAIL, P_REFDETAIL));
		}
	}
	
	public IPropertyDescriptor[] getPropertyDescriptors() {
		return (IPropertyDescriptor[]) descriptors.toArray(
                new IPropertyDescriptor[descriptors.size()]);
	}

	public Object getPropertyValue(Object id) {
		
		if (id.equals(P_ID_REFERENCE)) {
		
			ReferenceBase<?> reference = typeDesignation.getCitation();
			if (reference == null) {
				reference = Generic.NewInstance();
			}
			ReferencePropertySource referencePropertySource = new ReferencePropertySource(reference);
			referencePropertySource.addPropertyChangeListener(new PropertyChangeListener() {
				public void propertyChange(PropertyChangeEvent evt) {
					if (evt.getNewValue() instanceof ReferenceBase) {	
						typeDesignation.setCitation((ReferenceBase<?>) evt.getNewValue());
					}
				}
			});
			return referencePropertySource;
		}
		
		if (id.equals(P_ID_REFDETAIL)) {
			return CdmUtils.Nz(typeDesignation.getCitationMicroReference());
		}
		
		return null;
	}

	public boolean isPropertySet(Object id) {
		return false;
	}

	public void resetPropertyValue(Object id) {}

	public void setPropertyValue(Object id, Object value) {
		if (id.equals(P_ID_REFDETAIL)) {
			typeDesignation.setCitationMicroReference((String) value);
		}
	}
		
	public boolean equals(Object object) {
		if (object == null) {
			return false;
		}
		if (toString() == null) {
			return false;
		}
		if (toString().equals(object.toString())) {
			return true;
		} else {
			return false;
		}
	}
}
