/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation.handler;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoableOperation;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.TreeSelection;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.handlers.HandlerUtil;

import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.model.taxon.TaxonNode;
import eu.etaxonomy.taxeditor.dialogs.filteredSelection.FilteredTaxonNodeSelectionDialog;
import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.navigation.navigator.TaxonNavigator;
import eu.etaxonomy.taxeditor.operations.IPostOperationEnabled;
import eu.etaxonomy.taxeditor.operations.MoveTaxonOperation;

/**
 * @author n.hoffmann
 * @created 01.04.2009
 * @version 1.0
 */
public class MoveTaxonHandler extends AbstractHandler implements IPostOperationEnabled {
	private static final Logger logger = Logger
			.getLogger(MoveTaxonHandler.class);
	private TaxonNode parentTaxonNode;

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.IHandler#execute(org.eclipse.core.commands.ExecutionEvent)
	 */
	public Object execute(ExecutionEvent event) throws ExecutionException {
		TaxonNavigator taxonNavigator = NavigationUtil.getNavigator();
		
		TreeSelection selection = (TreeSelection) HandlerUtil.getCurrentSelection(event);
		
		Iterator selectionIterator = selection.iterator();
		Set<TaxonNode> taxonNodes = new HashSet<TaxonNode>();
		// do not show the current selection
		List<UUID> excludeTaxa = new ArrayList<UUID>();
		
	
		while (selectionIterator.hasNext()){
			Object object = selectionIterator.next();
			if(object instanceof TaxonNode){
				TaxonNode taxonNode = (TaxonNode) object;
				taxonNodes.add(taxonNode);
				excludeTaxa.add(taxonNode.getTaxon().getUuid());
			}
		}
		
//		TaxonNode taxonNode = (TaxonNode) selection.getFirstElement();
		
		parentTaxonNode = FilteredTaxonNodeSelectionDialog.selectTaxonNode(HandlerUtil.getActiveShell(event), "Choose new parent", excludeTaxa, null);
				
		if(parentTaxonNode != null){
			if(NavigationUtil.isDirty(parentTaxonNode)){
				MessageDialog.openWarning(HandlerUtil.getActiveShell(event), "Unsaved Parent Taxon", "There are unsaved " +
						"changes in the parent taxon. Please save first.");
				return null;
			}
			
			IUndoableOperation operation = new MoveTaxonOperation
					("Move taxon to new parent", NavigationUtil.getUndoContext(), 
							taxonNodes, parentTaxonNode, this, taxonNavigator); //$NON-NLS-1$
			NavigationUtil.executeOperation(operation);
		
		}
		
		return null;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.taxeditor.operations.IPostOperationEnabled#postOperation(eu.etaxonomy.cdm.model.common.CdmBase)
	 */
	public boolean postOperation(CdmBase objectAffectedByOperation) {
		return true;
	}
}
