// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.newWizards;

import org.apache.log4j.Logger;
import org.eclipse.core.commands.operations.AbstractOperation;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.ui.INewWizard;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.PartInitException;

import eu.etaxonomy.taxeditor.navigation.NavigationUtil;
import eu.etaxonomy.taxeditor.navigation.navigator.TaxonNavigator;
import eu.etaxonomy.taxeditor.operations.CreateTaxonomicTreeOperation;

/**
 * @author n.hoffmann
 * @created 23.06.2009
 * @version 1.0
 */
public class NewTaxonomicTreeWizard extends Wizard implements INewWizard {
	private static final Logger logger = Logger
			.getLogger(NewTaxonomicTreeWizard.class);

	public static final String NEW_TAXONOMIC_TREE = "NEW_TAXONOMIC_TREE";

	private NewTaxonomicTreeWizardPage page;
	
	/**
	 * 
	 */
	public NewTaxonomicTreeWizard() {
		this.setWindowTitle("New Taxonomic Tree");
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#addPages()
	 */
	@Override
	public void addPages() {
		super.addPages();
		
		page = new NewTaxonomicTreeWizardPage(NEW_TAXONOMIC_TREE);
		
		addPage(page);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.jface.wizard.Wizard#performFinish()
	 */
	@Override
	public boolean performFinish() {

		TaxonNavigator taxonNavigator;
		try {
			taxonNavigator = (TaxonNavigator) NavigationUtil.getView(TaxonNavigator.ID);
		} catch (PartInitException e) {
			logger.error(e);
			throw new RuntimeException(e);
		}
		
		AbstractOperation operation = new CreateTaxonomicTreeOperation("Create Taxonomic Tree", 
				NavigationUtil.getUndoContext(), page.getTreeLabel(), page.getReferenceUuid(), page.getMicroReference(),
				taxonNavigator, taxonNavigator);
		NavigationUtil.executeOperation(operation);

		NavigationUtil.getNavigator().getCommonViewer().refresh();
		
		return true;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.IWorkbenchWizard#init(org.eclipse.ui.IWorkbench, org.eclipse.jface.viewers.IStructuredSelection)
	 */
	public void init(IWorkbench workbench, IStructuredSelection selection) {
		
	}
}
