/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.operations;

import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.operations.IUndoContext;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;

import eu.etaxonomy.cdm.api.utility.ImagesUtility;
import eu.etaxonomy.cdm.model.description.CommonTaxonName;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Distribution;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.description.TaxonInteraction;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.cdm.model.taxon.Taxon;
import eu.etaxonomy.taxeditor.store.StoreUtil;

/**
 * @author p.ciardelli
 * @created 05.02.2009
 * @version 1.0
 * @author n.hoffmann
 */
public class CreateDescriptionElementOperation extends AbstractPostOperation {
	
	private TaxonDescription description;
	private Feature feature;
	private DescriptionElementBase element;

	public CreateDescriptionElementOperation(String label, IUndoContext undoContext,
			Taxon taxon, TaxonDescription description, Feature feature, IPostOperationEnabled postOperationEnabled) {
		super(label, undoContext, taxon, postOperationEnabled);
		
		this.description = description;
		this.feature = feature;
	}

	/**
	 * @param name
	 * @param undoContext
	 * @param taxon
	 * @param description
	 * @param feature
	 * @param element
	 * @param postOperationEnabled
	 */
	public CreateDescriptionElementOperation(String label,
			IUndoContext undoContext, Taxon taxon,
			TaxonDescription description, Feature feature,
			DescriptionElementBase element, IPostOperationEnabled postOperationEnabled) {
		this(label, undoContext, taxon, description, feature, postOperationEnabled);
		
		this.element = element;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#execute(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus execute(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {

		monitor.worked(20);
		if (element == null) {
			if (feature.isSupportsCommonTaxonName()) {
				element = CommonTaxonName.NewInstance("", null);
			} else if (feature.isSupportsDistribution()) {
				element = Distribution.NewInstance();
			} else if (feature.equals(Feature.IMAGE())){
				element = ImagesUtility.createImageElement(ImageFile.NewInstance(null, null));
			} else if(feature.isSupportsTaxonInteraction()){
				element = TaxonInteraction.NewInstance(feature);
			} else {
				element = TextData.NewInstance(feature);
			}
		}

		description.addElement(element);
		monitor.worked(40);

		return postExecute(element);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#redo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus redo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		description.addElement(element);
		
		return postExecute(element);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.core.commands.operations.AbstractOperation#undo(org.eclipse.core.runtime.IProgressMonitor, org.eclipse.core.runtime.IAdaptable)
	 */
	@Override
	public IStatus undo(IProgressMonitor monitor, IAdaptable info)
			throws ExecutionException {
		
		description.removeElement(element);
		
		return postExecute(null);
	}
}
